
// Author:  David Martin  <dmartin@eecs.berkeley.edu>

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import java.util.List;

public class ImageChooser 
    extends JDialog
    implements ActionListener
{
    int imageID = -1;
    List imageIDList;

    JList jList;
    JButton openButton, cancelButton;

    // Box with fixed height.
    class HBox extends Box {
	public HBox () {
	    super (BoxLayout.X_AXIS);
	}
	public Dimension getMaximumSize () {
	    Dimension maxSize = super.getMaximumSize();
	    Dimension minSize = super.getMinimumSize();
	    return new Dimension (maxSize.width, minSize.height);
	}
    };

    public ImageChooser (Frame owner, String title, List imageList) {
	super (owner, title, true);
	this.imageIDList = imageList;

	JLabel label = new JLabel ("Select an Image ID:");
	label.setForeground (Color.black);

	jList = new JList (imageIDList.toArray());
	jList.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
	jList.setSelectedIndex (0);
	JScrollPane scrollPane = new JScrollPane (
	    jList,
	    ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
	    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

	cancelButton = new JButton ("Cancel");
	openButton = new JButton ("Open");

	Box labelBox = new HBox (); 
	labelBox.add (Box.createHorizontalStrut (10));
	labelBox.add (label);
	labelBox.add (Box.createHorizontalGlue ());
	labelBox.add (Box.createHorizontalStrut (10));

	Box mainBox = Box.createHorizontalBox ();
	mainBox.add (Box.createHorizontalStrut (10));
	mainBox.add (scrollPane);
	mainBox.add (Box.createHorizontalStrut (10));

	Box buttonBox = new HBox ();
	buttonBox.add (Box.createHorizontalStrut (10));
	buttonBox.add (Box.createHorizontalGlue ());
	buttonBox.add (openButton);
	buttonBox.add (Box.createHorizontalStrut (10));
	buttonBox.add (cancelButton);
	buttonBox.add (Box.createHorizontalGlue ());
	buttonBox.add (Box.createHorizontalStrut (10));

	Box box = Box.createVerticalBox ();
	box.add (Box.createVerticalStrut (10));
	box.add (labelBox);
	box.add (Box.createVerticalStrut (5));
	box.add (mainBox);
	box.add (Box.createVerticalStrut (10));
	box.add (buttonBox);
	box.add (Box.createVerticalStrut (10));
	getContentPane().add (box);

	jList.addMouseListener (new MouseAdapter() {
	    public void mouseClicked (MouseEvent e) {
		if (e.getClickCount() == 2) {
		    int index = jList.getSelectedIndex();
		    if (index >= 0) { 
			imageID = ((Integer)imageIDList.get(index)).intValue();
			dispose ();
		    }
		}
	    }
	});

	cancelButton.addActionListener (this);
	openButton.addActionListener (this);

	setLocation (200, 200);
	pack ();
    }

    public void actionPerformed (ActionEvent e) {
	Object object = e.getSource ();
	if (object == cancelButton) {
	    dispose ();
	} else if (object == openButton) {
	    int index = jList.getSelectedIndex();
	    if (index >= 0) { 
		imageID = ((Integer)imageIDList.get(index)).intValue();
		dispose ();
	    }
	}
    }

    public int getImageID () {
	return imageID;
    }
}

