
// Author:  David Martin  <dmartin@eecs.berkeley.edu>

import java.util.*;

// Maintain a map of pixels-->segmentID.
// Also maintain a list of how many pixels each segment owns.
class ImageMask
    implements Cloneable
{
    public int 	width;
    public int 	height;

    int[][]	mask;
    Util.IntArray pixelCount;

    public ImageMask (int width, int height, int segID) {
	Util.assert (segID >= 0);
	this.width = width;
	this.height = height;
	this.mask = new int [height][width];
	if (segID != 0) {
	    for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
		    mask[y][x] = segID;
		}
	    }
	}
	pixelCount = new Util.IntArray ();
	pixelCount.set (segID, width * height);
    }

    public int pixelCount (int segID) {
	return pixelCount.get (segID);
    }

    public int nonEmptySegmentCount () {
	int count = 0;
	for (int i = 0; i < pixelCount.length(); i++) {
	    if (pixelCount.get(i) > 0) {
		count++;
	    }
	}
	return count;
    }

    public int[] nonEmptySegmentIDs () {
	int[] ids = new int [nonEmptySegmentCount()];
	int c = 0;
	for (int i = 0; i < pixelCount.length(); i++) {
	    if (pixelCount.get(i) > 0) {
		ids[c++] = i;
	    }
	}
	return ids;
    }

    public int get (int x, int y) {
	return mask[y][x];
    }

    public void set (int x, int y, int val) {
	pixelCount.set (mask[y][x], pixelCount.get (mask[y][x]) - 1);
	mask[y][x] = val;
	pixelCount.set (mask[y][x], pixelCount.get (mask[y][x]) + 1);
    }

    public int newSegmentID () {
	for (int id = 0; ; id++) {
	    if (pixelCount.get (id) == 0) {
		return id;
	    }
	}
    }
}

