
// Author:  David Martin  <dmartin@eecs.berkeley.edu>

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;
import java.net.*;
import gnu.regexp.*;

public class NewUser 
    extends JFrame 
    implements ActionListener
{
    static boolean superUser = false;

    static final String defaultImageIDServerHost = "ribbit.cs.berkeley.edu";
    static final int defaultImageIDServerPort = 2859;
    static String imageIDServerHost = defaultImageIDServerHost;
    static int imageIDServerPort = defaultImageIDServerPort;
    
    JTextField nameText, emailText;
    JButton cancelButton, submitButton;

    public NewUser () {
	super ("New User Registration");
	
	nameText = new JTextField (30);
	emailText = new JTextField (30);

	cancelButton = new JButton ("Cancel");
	submitButton = new JButton ("Submit");

	Box nameBox = Box.createHorizontalBox ();
	nameBox.add (Box.createHorizontalStrut (20));
	nameBox.add (new JLabel ("Name:"));
	nameBox.add (Box.createHorizontalStrut (5));
	nameBox.add (nameText);
	nameBox.add (Box.createHorizontalStrut (5));
	nameBox.add (Box.createHorizontalGlue ());

	Box emailBox = Box.createHorizontalBox ();
	emailBox.add (Box.createHorizontalStrut (20));
	emailBox.add (new JLabel ("Email Address:"));
	emailBox.add (Box.createHorizontalStrut (5));
	emailBox.add (emailText);
	emailBox.add (Box.createHorizontalStrut (5));
	emailBox.add (Box.createHorizontalGlue ());

	Box buttonBox = Box.createHorizontalBox ();
	buttonBox.add (Box.createHorizontalGlue ());
	buttonBox.add (cancelButton);
	buttonBox.add (Box.createHorizontalStrut (10));
	buttonBox.add (submitButton);
	buttonBox.add (Box.createHorizontalGlue ());
	
	nameText.setMaximumSize (nameText.getPreferredSize());
	emailText.setMaximumSize (emailText.getPreferredSize());

	Box mainBox = Box.createVerticalBox ();
	mainBox.add (Box.createVerticalGlue ());
	mainBox.add (Box.createVerticalStrut (20));
	mainBox.add (nameBox);
	mainBox.add (Box.createVerticalGlue ());
	mainBox.add (Box.createVerticalStrut (5));
	mainBox.add (emailBox);
	mainBox.add (Box.createVerticalGlue ());
	mainBox.add (Box.createVerticalStrut (20));
	mainBox.add (buttonBox);
	mainBox.add (Box.createVerticalGlue ());
	mainBox.add (Box.createVerticalStrut (20));

        this.getContentPane().add (mainBox);

	setDefaultCloseOperation (JFrame.DO_NOTHING_ON_CLOSE);
	addWindowListener (new WindowAdapter () {
	    public void windowClosing (WindowEvent e) {
		System.exit (0);
	    };
	});

	cancelButton.setActionCommand ("cancel");
	submitButton.setActionCommand ("submit");

	cancelButton.addActionListener (this);
	submitButton.addActionListener (this);

	setLocation (200, 200);
	pack();
    }

    public void actionPerformed (ActionEvent e) {
	String command = e.getActionCommand();
	if (command.equals ("cancel")) {
	    System.exit (0);
	} else if (command.equals ("submit")) {
	    submit ();
	} 
    }

    // Exits on success.  Returns on failure.
    public void submit () {
	String name = nameText.getText ();
	String email = emailText.getText ();
	RE re;

	// Clean up and verify name.
	try {
	    re = new RE ("\\s+"); 	name = re.substituteAll (name, " ");
	    re = new RE ("^ ");		name = re.substitute (name, "");
	    re = new RE (" $");		name = re.substitute (name, "");
	    if (name.length() == 0) {
		JOptionPane.showMessageDialog (
		    this, "Please specify your name.",
		    "Error", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    re = new RE ("^[ a-zA-Z.]+$");
	    if (!re.isMatch (name)) {
		JOptionPane.showMessageDialog (
		    this, "Name '" + name + "' contains illegal characters.",
		    "Error", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	} catch (REException e) {
	    JOptionPane.showMessageDialog (
		this, "Internal error: " + e.getMessage(),
		"Error", JOptionPane.ERROR_MESSAGE);
	    return;
	}

	// Clean up and verify email.
	try {
	    re = new RE ("\\s+"); 	email = re.substituteAll (email, " ");
	    re = new RE ("^ ");		email = re.substitute (email, "");
	    re = new RE (" $");		email = re.substitute (email, "");
	    if (email.length() == 0) {
		JOptionPane.showMessageDialog (
		    this, "Please specify your email address.",
		    "Error", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    re = new RE ("^[^@\\s]+@[^@\\s]+\\.[^@\\s]+$");
	    if (!re.isMatch (email)) {
		JOptionPane.showMessageDialog (
		    this, "Illegal email address '" + email + "'.",
		    "Error", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	} catch (REException e) {
	    JOptionPane.showMessageDialog (
		this, "Internal error: " + e.getMessage(),
		"Error", JOptionPane.ERROR_MESSAGE);
	    return;
	}

	int answer = JOptionPane.showConfirmDialog (
	    this, "Your name is \"" + name 
	    + "\" and your email is " + email 
	    + ".  Is this correct?",
	    "Question", JOptionPane.YES_NO_OPTION);
	if (answer == JOptionPane.NO_OPTION) {
	    return;
	}
	
	SegServer server = null;
	try {
	    // Connect to server.
	    server = new SegServer (imageIDServerHost, imageIDServerPort);
	    server.connect ();

	    // Send request.
            server.sendLine ("new user");
	    server.sendLine ("name " + name);
	    server.sendLine ("email " + email);
	    server.sendDone ();

	    // Process reply.
	    Map reply = server.getReply ();
	    int userID = Integer.parseInt ((String) reply.get ("userID"));
	    JOptionPane.showMessageDialog (
		this, "Your user ID is " + userID + ".",
		"Success", JOptionPane.INFORMATION_MESSAGE);
	    System.exit (0);

	} catch (Exception e) {
	    JOptionPane.showMessageDialog (
		this, "Error requesting new user ID: " + e.getMessage(), 
		"Error", JOptionPane.ERROR_MESSAGE);
	} finally {
	    if (server != null) { server.disconnect (); } 
	}
    }


    // Print usage and exit.
    static void usage () {
	System.err.print (
	    "usage: NewUser [options]\n" + 
	    "options:\n" +
	    "  -server <s>      Set server.\n" +
	    "                   [default " + defaultImageIDServerHost + "]\n" +
	    "  -port <n>        Set server port [default " + defaultImageIDServerPort + "].\n" +
	    "");
	System.exit (1);
    }

    static void processArgs (String args[]) {
	for (int i = 0; i < args.length; i++) {
	    if (args[i].equals ("-su")) {
		superUser = true;
	    } else if (args[i].equals ("-server")) {
		if (++i == args.length) { usage (); }
		imageIDServerHost = args[i];
	    } else if (args[i].equals ("-port")) {
		if (++i == args.length) { usage (); }
		try {
		    imageIDServerPort = Integer.decode(args[i]).intValue();
		} catch (NumberFormatException e) {
		    imageIDServerPort = -1;
		}
		if (imageIDServerPort < 0) { usage (); }
	    } else {
		usage ();
	    }
	}
    }

    public static void main (String[] args) {
	processArgs (args);
	NewUser app = new NewUser ();
	app.show ();
    }
}
